import pandas as pd
import numpy as np
import os

# TODO:
#     - fill NA when profile is missing in self.arr_options
#     - make consistency check for consumption and charging profiles


class Export:
    """docstring for Export."""

    def __init__(self, groupbyhr=True, rows=8760, kwpermw=1000):
        super(Export, self).__init__()
        self.groupbyhr = groupbyhr
        self.rows = rows
        self.kwpermw = kwpermw

    def loaddata(self, db):
        self.data = db

    def to_csv(self):

        self.code = []
        self.name = []
        self.consum = []
        self.availcap = []
        self.fleet = pd.DataFrame()

        self.listavailkeys = []
        self.k = 0
        for key in self.data.db.keys():
            if self.data.db[key]['kind'] == 'availability':
                self.k += 1
                self.listavailkeys.append(key)

        for i, key in enumerate(self.listavailkeys):
                df = self.data.db[key]['profile'][['hh', 'consumption', 'charging_cap']].copy()
                if self.groupbyhr:
                    df.loc[:, 'hh'] = df['hh'].astype(int)
                    df = df.groupby('hh').sum().reset_index()
                df = df[0:self.rows].copy()
                self.code.append(key)
                self.name.append('ev'+str(i+1))
                self.consum.append(df['consumption'].values / self.kwpermw)  # from kwh to MWh
                self.availcap.append(df['charging_cap'].values / self.kwpermw)  # from kwh to MWh

                self.fleet.loc[i, 'EV'] = 'ev'+str(i+1)
                self.fleet.loc[i, 'efficiency_charge'] = self.data.db[key]['charging_eff']
                self.fleet.loc[i, 'efficiency_discharge'] = 1.0
                self.fleet.loc[i, 'ev_start'] = self.data.db[key]['soc_init']
                self.fleet.loc[i, 'ev_capacity'] = self.data.db[key]['battery_capacity'] / self.kwpermw  # from kwh to MWh
                self.fleet.loc[i, 'share_ev'] = 1/self.k
                self.fleet.loc[i, 'cons_rate'] = self.data.db[key]['energy_consumption'] / self.kwpermw  # MWh per km
                self.fleet.loc[i, 'Passed'] = self.data.db[key]['success']
                self.fleet.loc[i, 'info'] = [self.data.db[key]['description']]
                self.fleet.loc[i, 'code'] = key

        self.arr_cons = np.array(self.consum)
        self.arr_avai = np.array(self.availcap)

        self.consdf = pd.DataFrame(self.arr_cons.T, columns=self.name)
        self.consdf.columns = pd.MultiIndex.from_product([['Demand_MWh'], ['ev_ed'], self.consdf.columns])

        self.avaidf = pd.DataFrame(self.arr_avai.T, columns=self.name)
        self.avaidf.columns = pd.MultiIndex.from_product([['Grid_connect_MW'], ['n_ev_p'], self.avaidf.columns])
        self.final = self.consdf.join(self.avaidf)

        self.subscen = {}
        self.setopt = set()

        for cd in self.code:
            temp = {}
            for key in self.data.db.keys():
                if self.data.db[key]['kind'] != 'driving':
                    if self.data.db[key]['input'] == cd:
                        option = self.data.db[key]['option']
                        temp[option] = key
                        self.setopt.add(option)
            if temp:
                self.subscen[cd] = temp

        if self.subscen:
            self.optdict = dict(zip(self.setopt, range(len(self.setopt))))
            self.arr_options = np.empty((len(self.setopt), len(self.code), self.rows))
            for id1, cd in enumerate(self.code):
                if self.subscen[cd]:
                    for key, value in self.subscen[cd].items():
                        id0 = self.optdict[key]
                        df = self.data.db[value]['profile'][['hh', 'charge_grid']].copy()
                        if self.groupbyhr:
                            df.loc[:, 'hh'] = df['hh'].astype(int)
                            df = df.groupby('hh').sum().reset_index()
                        df = df[0:self.rows].copy()
                        self.arr_options[id0, id1, :] = df['charge_grid'].values / self.kwpermw
            self.arr_options = self.arr_options.round(7)
            self.frame = pd.DataFrame(index=range(self.rows), columns=pd.MultiIndex.from_product([[], [], []]))
            for opt, idopt in self.optdict.items():
                dfop = pd.DataFrame(self.arr_options[idopt].T, columns=self.name)
                dfop.columns = pd.MultiIndex.from_product([[opt + '_MWh'], ['ev_ged_exog'], dfop.columns])
                self.frame = self.frame.join(dfop)
            self.final = self.final.join(self.frame)

        self.final.loc[:, 'Hour'] = ['h' + str(j+1) for j in range(self.rows)]
        self.final.set_index('Hour', inplace=True)

    def save_files(self, repository=""):
        if repository:
            self.repository = repository
        else:
            self.repository = self.data.folder
        os.makedirs(self.repository, exist_ok=True)
        self.final.to_csv(os.path.join(self.repository, '0time_series.csv'))
        self.fleet.to_csv(os.path.join(self.repository, '0data_input.csv'), index=False)
